/**
 * Marlin 3D Printer Firmware
 * Copyright (C) 2016 MarlinFirmware [https://github.com/MarlinFirmware/Marlin]
 *
 * Based on Sprinter and grbl.
 * Copyright (C) 2011 Camiel Gubbels / Erik van der Zalm
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

// R25 = 100 kOhm, beta25 = 3974 K, 4.7 kOhm pull-up, Honeywell 135-104LAF-J01
// R0 = 100000 Ohm
// T0 = 25 °C
// Beta = 3974
// R1 = 0 Ohm
// R2 = 4700 Ohm
const short temptable_71[][2] PROGMEM = {
  { OV(  35), 300 },
  { OV(  51), 270 },
  { OV(  54), 265 },
  { OV(  58), 260 },
  { OV(  59), 258 },
  { OV(  61), 256 },
  { OV(  63), 254 },
  { OV(  64), 252 },
  { OV(  66), 250 },
  { OV(  67), 249 },
  { OV(  68), 248 },
  { OV(  69), 247 },
  { OV(  70), 246 },
  { OV(  71), 245 },
  { OV(  72), 244 },
  { OV(  73), 243 },
  { OV(  74), 242 },
  { OV(  75), 241 },
  { OV(  76), 240 },
  { OV(  77), 239 },
  { OV(  78), 238 },
  { OV(  79), 237 },
  { OV(  80), 236 },
  { OV(  81), 235 },
  { OV(  82), 234 },
  { OV(  84), 233 },
  { OV(  85), 232 },
  { OV(  86), 231 },
  { OV(  87), 230 },
  { OV(  89), 229 },
  { OV(  90), 228 },
  { OV(  91), 227 },
  { OV(  92), 226 },
  { OV(  94), 225 },
  { OV(  95), 224 },
  { OV(  97), 223 },
  { OV(  98), 222 },
  { OV(  99), 221 },
  { OV( 101), 220 },
  { OV( 102), 219 },
  { OV( 104), 218 },
  { OV( 106), 217 },
  { OV( 107), 216 },
  { OV( 109), 215 },
  { OV( 110), 214 },
  { OV( 112), 213 },
  { OV( 114), 212 },
  { OV( 115), 211 },
  { OV( 117), 210 },
  { OV( 119), 209 },
  { OV( 121), 208 },
  { OV( 123), 207 },
  { OV( 125), 206 },
  { OV( 126), 205 },
  { OV( 128), 204 },
  { OV( 130), 203 },
  { OV( 132), 202 },
  { OV( 134), 201 },
  { OV( 136), 200 },
  { OV( 139), 199 },
  { OV( 141), 198 },
  { OV( 143), 197 },
  { OV( 145), 196 },
  { OV( 147), 195 },
  { OV( 150), 194 },
  { OV( 152), 193 },
  { OV( 154), 192 },
  { OV( 157), 191 },
  { OV( 159), 190 },
  { OV( 162), 189 },
  { OV( 164), 188 },
  { OV( 167), 187 },
  { OV( 170), 186 },
  { OV( 172), 185 },
  { OV( 175), 184 },
  { OV( 178), 183 },
  { OV( 181), 182 },
  { OV( 184), 181 },
  { OV( 187), 180 },
  { OV( 190), 179 },
  { OV( 193), 178 },
  { OV( 196), 177 },
  { OV( 199), 176 },
  { OV( 202), 175 },
  { OV( 205), 174 },
  { OV( 208), 173 },
  { OV( 212), 172 },
  { OV( 215), 171 },
  { OV( 219), 170 },
  { OV( 237), 165 },
  { OV( 256), 160 },
  { OV( 300), 150 },
  { OV( 351), 140 },
  { OV( 470), 120 },
  { OV( 504), 115 },
  { OV( 538), 110 },
  { OV( 552), 108 },
  { OV( 566), 106 },
  { OV( 580), 104 },
  { OV( 594), 102 },
  { OV( 608), 100 },
  { OV( 622),  98 },
  { OV( 636),  96 },
  { OV( 650),  94 },
  { OV( 664),  92 },
  { OV( 678),  90 },
  { OV( 712),  85 },
  { OV( 745),  80 },
  { OV( 758),  78 },
  { OV( 770),  76 },
  { OV( 783),  74 },
  { OV( 795),  72 },
  { OV( 806),  70 },
  { OV( 818),  68 },
  { OV( 829),  66 },
  { OV( 840),  64 },
  { OV( 850),  62 },
  { OV( 860),  60 },
  { OV( 870),  58 },
  { OV( 879),  56 },
  { OV( 888),  54 },
  { OV( 897),  52 },
  { OV( 905),  50 },
  { OV( 924),  45 },
  { OV( 940),  40 },
  { OV( 955),  35 },
  { OV( 967),  30 },
  { OV( 970),  29 },
  { OV( 972),  28 },
  { OV( 974),  27 },
  { OV( 976),  26 },
  { OV( 978),  25 },
  { OV( 980),  24 },
  { OV( 982),  23 },
  { OV( 984),  22 },
  { OV( 985),  21 },
  { OV( 987),  20 },
  { OV( 995),  15 },
  { OV(1001),  10 },
  { OV(1006),   5 },
  { OV(1010),   0 }
};
